/*
 * GA.java
 *
 * Created on 7 de febrero de 2005, 19:38
 */

package geneticalgorithms.GAStructures;
import geneticalgorithms.StopCriterions.StopCriterion;
import geneticalgorithms.Reporters.Reporter;
import geneticalgorithms.Initializers.Initializer;
import geneticalgorithms.ChromosomeRepresentations.Chromosome;
import FitnessFunctions.FitnessFunction;
import java.util.Vector;

/**
 *
 * @author Carlos
 */
public abstract class SOGA implements GA {
  
  protected FitnessFunction fitnessFunction;
  protected StopCriterion stopCriterion;
  protected Reporter reporter;
  protected Vector population;
  protected int numChromosomes;
  protected Initializer initializer;
  protected int numIterations;
  
  /** Creates a new instance of GA */
  public SOGA(FitnessFunction fitnessFunction, Initializer initializer,
       StopCriterion stopCriterion, int numChromosomes, Reporter reporter) {
    
    int i;
    this.stopCriterion = stopCriterion;
    this.reporter = reporter;
    this.initializer = initializer;
    this.population = new Vector();
    this.numChromosomes = numChromosomes;
    this.fitnessFunction = fitnessFunction;
    
    initializer.initialize(this.population, numChromosomes);
    
    for (i = 0; i < numChromosomes; i++){
      Chromosome aChromosome = (Chromosome) population.get(i);
      aChromosome.setCurrentFitness(fitnessFunction.fitness(aChromosome, this.population));
    }
  }
  
  public SOGA(FitnessFunction fitnessFunction, Vector population, StopCriterion stopCriterion,
       Reporter reporter){
    
    this.fitnessFunction = fitnessFunction;
    this.reporter = reporter;
    this.stopCriterion = stopCriterion;
    this.population = population;
    this.initializer = null;
    this.numChromosomes = population.size();
  }
  
  /** This function defines an iteration of the algorithm. */
  public abstract void iterate();
  
  /** This function returns the population of the GA. */
  public Vector getPopulation(){
    return this.population;
  }
  
  /** This function returns the number of evaluations. */
  public long getNumEvaluations(){
    return (this.fitnessFunction.getNumEvaluations());
  }
  
  /** returns the best fitness. */
  public double getBestFitness(){
    
    int i;
    
    double bestFitness, currentFitness;
    Chromosome aChromosome = (Chromosome) population.get(0);
    bestFitness = aChromosome.getCurrentFitness();
    
    for (i = 1; i < numChromosomes; i++){
      aChromosome = (Chromosome) population.get(i);
      currentFitness = aChromosome.getCurrentFitness();
      
      if (this.fitnessFunction.compare(currentFitness, bestFitness) > 0)
        bestFitness = currentFitness;
    }
    
    return bestFitness;
  }
  
  /** returns the best chromosome. */
  public Chromosome getBestChromosome(){
    
    int i;
    
    double bestFitness, currentFitness;
    Chromosome bestChromosome;
    Chromosome aChromosome = (Chromosome) population.get(0);
    bestFitness = aChromosome.getCurrentFitness();
    bestChromosome = aChromosome;
    
    for (i = 1; i < numChromosomes; i++){
      aChromosome = (Chromosome) population.get(i);
      currentFitness = aChromosome.getCurrentFitness();
      
      if (this.fitnessFunction.compare(currentFitness, bestFitness) > 0){
        bestFitness = currentFitness;
        bestChromosome = aChromosome;
      }
    }
    
    return bestChromosome;
  }
  
  /** This function run the GA until the Stop Criterion is reached. */
  public void run(){
    
    numIterations = 0;
    
    while (!this.stopCriterion.isTrue(this)){
      
      if (reporter != null && reporter.needReport(this))
        reporter.report(this);
      
      this.iterate();
      numIterations++;
    }
  }
  
  public long getNumIterations(){
    
    return numIterations;
  }
}
